@@ -1,3 +1,4 @@
+# -*- coding: utf8 -*-
 #!/usr/bin/env python
 """The main entry point. Invoke as `undatum' or `python -m undatum`.
 
@@ -7,12 +8,12 @@ import sys
 
 def main():
     try:
-        from .core import cli
-        exit_status = cli()
+        from .core import app
+        app()
     except KeyboardInterrupt:
         print("Ctrl-C pressed. Aborting")
     sys.exit(0)
 
 
 if __name__ == '__main__':
-    main()
+    app()
